import 'dart:io';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';

class DpImageWidget extends StatelessWidget {
  final String? placeholderAsset;
  final IconData? placeholderIcon;
  final String? imageUrl;
  final File? imageFile;
  final void Function()? editCallback;
  final bool hasError;
  final bool editable;
  final IconData? editIcon;
  final double frameRadius;
  final bool isCircular;
  final double borderRadius;
  final double? height;
  final double? width;
  final Widget Function(BuildContext)? editButtonBuilder;

  const DpImageWidget({
    Key? key,
    this.placeholderAsset,
    this.placeholderIcon,
    this.imageUrl,
    this.imageFile,
    this.editCallback,
    this.hasError = false,
    this.editable = true,
    this.editIcon,
    this.frameRadius = 45.0,
    this.isCircular = true,
    this.borderRadius = 12,
    this.height,
    this.width,
    this.editButtonBuilder,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        Builder(builder: (context) {
          final frameDecoration = BoxDecoration(
            color: Colors.grey,
            shape: isCircular ? BoxShape.circle : BoxShape.rectangle,
            borderRadius:
                !isCircular ? BorderRadius.circular(borderRadius) : null,
          );
          final _PlaceHolder placeholder = _PlaceHolder(
            isCircular: isCircular,
            borderRadius: borderRadius,
            placeholderAsset: placeholderAsset,
            height: height,
            frameRadius: frameRadius,
            width: width,
          );

          if (imageFile != null) {
            return Container(
              clipBehavior: Clip.hardEdge,
              decoration: BoxDecoration(
                color: Colors.grey,
                shape: isCircular ? BoxShape.circle : BoxShape.rectangle,
                borderRadius:
                    !isCircular ? BorderRadius.circular(borderRadius) : null,
              ),
              height: height ?? frameRadius * 2,
              width: width ?? frameRadius * 2,
              child: Image.file(
                imageFile!,
                fit: BoxFit.cover,
              ),
            );
          } else if (imageUrl == null) {
            return placeholder;
          } else {
            return CachedNetworkImage(
              imageUrl: imageUrl!,
              imageBuilder: (context, imageProvider) {
                return Container(
                  clipBehavior: Clip.hardEdge,
                  decoration: frameDecoration,
                  height: height ?? frameRadius * 2,
                  width: width ?? frameRadius * 2,
                  child: Image(
                    image: imageProvider,
                    fit: BoxFit.cover,
                  ),
                );
              },
              placeholder: (context, url) => placeholder,
              errorWidget: (context, url, error) => placeholder.copyWith(
                hasError: true,
                errorMessage:
                    (error is HandshakeException) ? "Connection Error" : null,
              ),
            );
          }
        }),
        if (editable)
          Positioned(
            bottom: 4,
            right: 4,
            child: GestureDetector(
              onTap: editCallback,
              child: editButtonBuilder != null
                  ? editButtonBuilder!(context)
                  : Container(
                      decoration: BoxDecoration(
                        shape:
                            isCircular ? BoxShape.circle : BoxShape.rectangle,
                        color: hasError
                            ? Colors.grey.shade100
                            : Theme.of(context).indicatorColor,
                        borderRadius:
                            !isCircular ? BorderRadius.circular(8) : null,
                      ),
                      // height: 35,
                      // width: 35,
                      child: Padding(
                        padding: const EdgeInsets.all(4.0),
                        child: Icon(
                          editIcon ?? Icons.edit_sharp,
                          color: hasError ? Colors.red : Colors.white,
                          size: 18,
                        ),
                      ),
                    ),
            ),
          ),
      ],
    );
  }
}

class _PlaceHolder extends StatelessWidget {
  const _PlaceHolder({
    Key? key,
    required this.isCircular,
    required this.borderRadius,
    this.placeholderAsset,
    this.placeholderIcon,
    required this.height,
    required this.frameRadius,
    required this.width,
    this.hasError = false,
    this.errorMessage,
  }) : super(key: key);

  _PlaceHolder copyWith({bool hasError = false, String? errorMessage}) {
    return _PlaceHolder(
      isCircular: isCircular,
      borderRadius: borderRadius,
      placeholderAsset: placeholderAsset,
      height: height,
      frameRadius: frameRadius,
      width: width,
      hasError: hasError,
      errorMessage: errorMessage,
    );
  }

  final bool isCircular;
  final double borderRadius;
  final String? placeholderAsset;
  final IconData? placeholderIcon;
  final double? height;
  final double frameRadius;
  final double? width;
  final bool hasError;
  final String? errorMessage;

  @override
  Widget build(BuildContext context) {
    return Container(
      clipBehavior: Clip.hardEdge,
      decoration: BoxDecoration(
          color: Colors.transparent,
          shape: isCircular ? BoxShape.circle : BoxShape.rectangle,
          borderRadius:
              !isCircular ? BorderRadius.circular(borderRadius) : null,
          image: placeholderAsset != null
              ? DecorationImage(
                  image: AssetImage(placeholderAsset!),
                  colorFilter: hasError
                      ? ColorFilter.mode(
                          Colors.red.withOpacity(0.4), BlendMode.modulate)
                      : null,
                  fit: BoxFit.fill)
              : null),
      height: height ?? frameRadius * 2,
      width: width ?? frameRadius * 2,
      child: Builder(builder: (context) {
        if (hasError) {
          return Center(
            child: Padding(
              padding: const EdgeInsets.all(4.0),
              child: Text(
                errorMessage ?? "An Error Occurred",
                style: const TextStyle(color: Colors.red),
              ),
            ),
          );
        } else {
          if (placeholderAsset != null) {
            return Container();
          }
          return CircleAvatar(
            child: Icon(
              Icons.person,
              size: frameRadius,
            ),
          );
        }
      }),
    );
  }
}
